
////////////////////////////////////////////////////////////////////////////
//                   PaneMultiTab by Elm (www.netcult.ch/elmue)
////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PaneMultiTab.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

CPaneMultiTab::CPaneMultiTab()
{
}

CPaneMultiTab::~CPaneMultiTab()
{
}

BEGIN_MESSAGE_MAP(CPaneMultiTab, CXTPMDIFrameWnd)
	//{{AFX_MSG_MAP(CPaneMultiTab)
	ON_WM_CREATE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

int CPaneMultiTab::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CXTPMDIFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	// required to set the theme correctly and for the ShowActiveFiles drop down button to work correctly
	if (!InitCommandBars())
		return -1;

	VERIFY(mi_TabbedWorkspace.Attach(this, TRUE));

	mi_TabbedWorkspace.GetPaintManager()->m_bShowIcons = FALSE;
	mi_TabbedWorkspace.EnableToolTips();
	mi_TabbedWorkspace.SetFlags(xtpWorkspaceShowActiveFiles);
	return 0;
}

BOOL CPaneMultiTab::OnCreateClient(LPCREATESTRUCT lpcs, CCreateContext* p_Context)
{
	// Avoid ASSERT because of missing menu items
	return CreateClient(lpcs, NULL);
}

// s_Path = "C:\Temp\Test.script" opens a file in a new Tab
// s_Path = NULL opens a new empty Tab
// Use ActivateFile() to check if the file is already open!
// See CMultiDocTemplate::OpenDocumentFile()
CDocument* CPaneMultiTab::AddNewTab(LPCTSTR s_Path,
									UINT  u32_ResourceID,
									CRuntimeClass* pi_DocClass,   // e.g. CScriptEditorDoc
                                    CRuntimeClass* pi_ViewClass,  // e.g. CScriptEditorView
									CRuntimeClass* pi_FrameClass, // e.g. CChildFrame
									BOOL  b_MakeVisible) // =TRUE
{
	CDocument* pi_Doc   = NULL;
	CFrameWnd* pi_Frame = NULL;
	CCreateContext i_Context;

	// the pi_View instance will be deleted in PostNcDestroy
	pi_Doc = (CDocument*)pi_DocClass->CreateObject();
	if (!pi_Doc) 
		goto _Error;

	ASSERT_VALID(pi_Doc);

	pi_Doc->m_bAutoDelete = FALSE; // don't destroy if something goes wrong

	pi_Frame = (CFrameWnd*)pi_FrameClass->CreateObject();
	if (!pi_Frame) 
		goto _Error;

	ASSERT_KINDOF(CFrameWnd, pi_Frame);

	i_Context.m_pNewViewClass   = pi_ViewClass;
	i_Context.m_pCurrentDoc     = pi_Doc;
	i_Context.m_pNewDocTemplate = &mi_DocList; // mi_DocList.m_docList stores all documents
	i_Context.m_pCurrentFrame   = NULL;

	if (!pi_Frame->LoadFrame(u32_ResourceID, WS_OVERLAPPEDWINDOW|FWS_ADDTOTITLE, this, &i_Context))
		goto _Error;

	if (s_Path != NULL) // Open document in new Tab
	{
		CWaitCursor i_Wait;
		if (!pi_Doc->OnOpenDocument(s_Path))
			goto _Error;

		// Param2=TRUE -> Add to MRU list (Recent File List)
		pi_Doc->SetPathName(s_Path, TRUE);
	}
	else // Open empty Tab
	{
		if (!b_MakeVisible) pi_Doc->m_bEmbedded = TRUE;

		mi_DocList.SetDefaultTitle(pi_Doc);

		if (!pi_Doc->OnNewDocument())
			goto _Error;
	}

	pi_Frame->InitialUpdateFrame(pi_Doc, b_MakeVisible);

	// Delete the document instance when pi_Doc->OnCloseDocument() is called
	pi_Doc->m_bAutoDelete = TRUE;

	// Store in the document list. If a document gets closed it is removed automatically from this list.
	mi_DocList.AddDocument(pi_Doc);
	return pi_Doc;

_Error:
	ASSERT(0);

	if (pi_Frame)
	{
		pi_Frame->DestroyWindow();
		delete pi_Frame;
	}
	if (pi_Doc)
	{
		delete pi_Doc;
	}
	return NULL;
}

CView* CPaneMultiTab::GetActiveView()
{
	CFrameWnd* pi_Frame = MDIGetActive();
	if (!pi_Frame)
		return NULL;

	// The first child of the Frame is always the View
	return (CView*)pi_Frame->GetTopWindow();
}

CDocument* CPaneMultiTab::GetActiveDoc()
{
	CView* pi_View = GetActiveView();
	if (!pi_View)
		return NULL;

	return pi_View->GetDocument();
}

void CPaneMultiTab::RemoveDoc(CDocument* pi_Doc)
{
	if (pi_Doc)
		mi_DocList.RemoveDocument(pi_Doc);
}

POSITION CPaneMultiTab::GetFirstDocPosition() const
{
	return mi_DocList.GetFirstDocPosition();
}

CDocument* CPaneMultiTab::GetNextDoc(POSITION& Pos) const
{
	return mi_DocList.GetNextDoc(Pos);
}

int CPaneMultiTab::GetDocCount()
{
	return mi_DocList.GetCount();
}

// Activate the Tab of an open file and returns TRUE,
// or returns FALSE when the file is not open
// Call this before calling AddNewTab() to avoid that a file gets opened twice
BOOL CPaneMultiTab::ActivateFile(CString s_Path)
{
	CDocument* pi_Doc = GetDocByPath(s_Path);
	if (!pi_Doc)
		return FALSE;

	return ActivateDoc(pi_Doc);
}

CDocument* CPaneMultiTab::GetDocByPath(CString s_Path)
{
	if (!s_Path.GetLength())
		return NULL;

	POSITION Pos = mi_DocList.GetFirstDocPosition();
	while (Pos)
	{
		CDocument* pi_Doc = (CDocument*) mi_DocList.GetNextDoc(Pos);
		if (pi_Doc && s_Path.CompareNoCase(pi_Doc->GetPathName()) == 0)
		{
			return pi_Doc;
		}
	}
	return NULL;
}

// Activate the Tab which contains the given document
BOOL CPaneMultiTab::ActivateDoc(CDocument* pi_Doc)
{
	if (!pi_Doc)
		return FALSE;

	POSITION Pos = pi_Doc->GetFirstViewPosition();
	if (!Pos)
		return FALSE;
	
	CView *pi_View = pi_Doc->GetNextView(Pos);
	if (!pi_View)
		return FALSE;
		
	CFrameWnd* pi_Frame = pi_View->GetParentFrame();
	if (!pi_Frame)
		return FALSE;

	pi_Frame->ActivateFrame();
	return TRUE;
}

BOOL CPaneMultiTab::SaveAndClose(CDocument* pi_Doc)
{
	if (!pi_Doc)
		return FALSE;

	if (pi_Doc->IsModified())
	{
		if (!pi_Doc->DoFileSave())
			return FALSE;
	}
	pi_Doc->OnCloseDocument();
	return TRUE;
}

BOOL CPaneMultiTab::SaveAll() 
{
	POSITION Pos = mi_DocList.GetFirstDocPosition();
	while (Pos)
	{
		CDocument* pi_Doc = mi_DocList.GetNextDoc(Pos);
		if (pi_Doc->IsModified())
		{
			if (!pi_Doc->DoFileSave())
				return FALSE;
		}
	}
	return TRUE;
}

// Call this function from CMainFrame::OnClose() !
BOOL CPaneMultiTab::CloseAll() 
{
	POSITION Pos = mi_DocList.GetFirstDocPosition();
	while (Pos)
	{
		CDocument* pi_Doc = mi_DocList.GetNextDoc(Pos);
		if (!SaveAndClose(pi_Doc))
			return FALSE;
	}
	return TRUE;
}
